import React, { Component } from 'react';
import { Modal, Tabs, Alert } from 'antd';
import LoginForm from './LoginForm';
import fetch from 'isomorphic-fetch';
import queryString from 'query-string';

const appid = 'dingoaodhkaxn4sviutdf9';
const redirectUrl = encodeURIComponent(window.location.protocol + '//sso.cathay-ins.com.cn/sso/dingding/callback');
const GOTO_URL = `https://oapi.dingtalk.com/connect/oauth2/sns_authorize?appid=${appid}&response_type=code&scope=snsapi_login&state=STATE&redirect_uri=${redirectUrl}`;

class LoginModal extends Component {
  state = { visible: false };

  constructor({ callbackUrl }) {
    super();
    // 用在页面版单点登陆中
    this.callbackUrl = callbackUrl;
    this.handleIframeMessage = this.handleIframeMessage.bind(this);

    if (typeof window.addEventListener !== 'undefined') {
      window.addEventListener('message', this.handleIframeMessage, false);
    } else if (typeof window.attachEvent !== 'undefined') {
      window.attachEvent('onmessage', this.handleIframeMessage);
    }
  }

  show(lastFetch) {
    this.lastFetch = lastFetch;
    this.setState({
      visible: true
    });
  }

  close() {
    this.setState({
      visible: false
    });
  }

  receiveTmpAuthCodeIframe(url) {
    var iframe = document.createElement('iframe');
    iframe.src = url;
    iframe.frameBorder = '0';
    iframe.allowTransparency = 'true';
    iframe.scrolling = 'no';

    document.body.appendChild(iframe);
    return iframe;
  }

  doAuthByFetch(type, data) {
    const lastFetch = this.lastFetch;
    if (!lastFetch) return;

    let url = lastFetch.url;
    let options = lastFetch.options;
    let headers = options.headers;

    switch (type) {
      case 'ddScan':
        // 兼容方案 start，去掉之前请确定所有java sdk等升级到指定版本
        headers['CATHAY_AUTH_TYPE'] = 'ddScan';
        headers['CATHAY_AUTH_DD_TMP_CODE'] = data;
        // 兼容方案 end

        headers['CATHAY-AUTH-TYPE'] = 'ddScan';
        headers['CATHAY-AUTH-DD-TMP-CODE'] = data;
        break;
      case 'password':
        // 兼容方案 start，去掉之前请确定所有java sdk等升级到指定版本
        headers['CATHAY_AUTH_TYPE'] = 'password';
        headers['CATHAY_AUTH_USERNAME'] = encodeURIComponent(data.username);
        headers['CATHAY_AUTH_PASSWORD'] = data.password;
        // 兼容方案 end

        headers['CATHAY-AUTH-TYPE'] = 'password';
        headers['CATHAY-AUTH-USERNAME'] = encodeURIComponent(data.username);
        headers['CATHAY-AUTH-PASSWORD'] = data.password;
        break;
      default:
        break;
    }

    this.doVerify(url, options, type);
  }

  fetchCanBeRecover() {
    const lastFetch = this.lastFetch;

    const minite10 = 10 * 60 * 1000;

    // 如果距离上个请求的时间在10min以上，那么判断为页面初始化后的单次数据请求，直接恢复数据
    if (lastFetch.intervalTime > minite10) return true;

    // 累计fetch请求数 < 10 认为是首次加载页面，暂无想到更严谨的判断方法
    // @TODO
    return lastFetch.count >= 10
  }

  doVerify(url, options, type) {
    fetch(url, options)
      .then(res => {
        if (res.status === 401 || res.status === 400) {
          res.json().then(json => {
            let message = json.message || '账号或密码错误，请重试';

            let modal = Modal.error({
              title: '登陆失败及提醒',
              width: 600,
              content: (
                <div>
                  <p style={{ margin: '20px 0' }}>{message}</p>
                  <Alert
                    message="原因提醒: 不是老核心密码！"
                    description={
                      <p>
                        新核心系统的密码与老核心系统密码不一样，分两种情况
                        <ul>
                          <li>
                            1.
                            对于新员工：密码会通过短信或邮箱在入职当天发送给员工
                          </li>
                          <li>
                            2. 对于老员工：密码为2017年最后一天老核心的密码
                          </li>
                        </ul>
                        更详细的说明，可以参见：
                        <a
                          href="http://bbs.cathay-ins.com.cn/topic/5a532afd96c542bf3f18bc99"
                          target="_blank"
                        >
                          关于国泰各系统的账号、密码说明
                        </a>
                        <br />
                        <br />
                        <strong>
                          如果确实忘记密码，可以通过`密码找回`功能重置密码
                        </strong>
                        <img
                          alt=""
                          width="320"
                          src="http://cathay-cms-prod.oss-cn-hzfinance-internal.aliyuncs.com/prod/posts/ceb9803b0986e057e8abd891d358aeae.png"
                        />
                      </p>
                    }
                    type="warning"
                    showIcon
                  />
                </div>
              )
            });

            setTimeout(() => {
              modal.destroy();
            }, 3000);
          });
        } else if (res.status === 200) {
          res
            .json()
            .then(json => {
              return new Promise((resolve, reject) => {
                window.hufu.risk({
                  fast: true,
                  uId: json.data.uid,
                  token: json.data.caToken,
                  callback: resolve
                });
              });
            })
            .then(() => {
              this.close();

              // 用在页面版单点登陆中
              if (this.callbackUrl) {
                Modal.success({
                  title: '登陆成功',
                  content: '登陆成功，页面会即将跳转'
                });

                setTimeout(() => {
                  window.location.href = this.callbackUrl;
                }, 500);

                // 用在Modal版单点登陆中
              } else if (this.fetchCanBeRecover()) {
                let modal = Modal.success({
                  title: '登陆成功',
                  content: '重新登陆成功，请重新完成个之前的操作'
                });
                setTimeout(() => {
                  modal.destroy();
                }, 500);
                // 用在Modal版单点登陆中
              } else {
                Modal.success({
                  title: '登陆成功',
                  content: '登陆成功，页面即将刷新'
                });

                setTimeout(() => {
                  window.location.reload();
                }, 500);
              }
            });
        } else {
          return Promise.reject(new Error('uncaught error'));
        }
      })
      .catch(err => {
        if (err) {
          let modal = Modal.error({
            title: '登陆失败',
            content: '登陆异常，请重试，本窗口3s消失'
          });
          setTimeout(() => {
            modal.destroy();
          }, 3000);
        }
      });
  }

  handleIframeMessage(event) {
    // 参考: https://open-doc.dingtalk.com/docs/doc.htm?treeId=168&articleId=104882&docType=1
    if (event.origin === 'https://login.dingtalk.com') {
      // 拿到loginTmpCode后就可以在这里构造跳转链接进行跳转了
      var loginTmpCode = event.data;
      this.iframe = this.receiveTmpAuthCodeIframe(
        `${GOTO_URL}&loginTmpCode=${loginTmpCode}`
      );
    } else if (typeof event.data === 'string') {
      var query = queryString.parse(event.data);
      if (query.code) this.doAuthByFetch('ddScan', query.code);
    }
  }

  renderScanCode() {
    window.DDLogin({
      id: 'scanCode',
      goto: encodeURIComponent(GOTO_URL),
      width: '300px',
      height: '300px'
    });
  }

  render() {
    const TabPane = Tabs.TabPane;

    return (
      <Modal
        title="国泰统一登陆"
        visible={this.state.visible}
        onCancel={() => {
          this.close();
        }}
        footer=""
      >
        <Tabs type="card">
          <TabPane tab="扫码登录" key="1">
            <div
              id="scanCode"
              style={{
                width: 300,
                height: 300,
                marginBottom: 30,
                marginLeft: 80
              }}
            />
            <div className="sso-tips">
              国泰员工默认绑定了钉钉扫码，其他账号需要登陆后自己绑定:{' '}
              <a href="//home.cathay-ins.com.cn/account/bind-dd">绑定钉钉</a>
            </div>
          </TabPane>

          <TabPane tab="输入登录" key="2">
            <LoginForm
              doLogin={data => {
                this.doAuthByFetch('password', data);
              }}
            />
            <div className="sso-tips">
              忘记密码？
              <a href="//home.cathay-ins.com.cn/nologin/password/recover">
                找回密码
              </a>
              &nbsp;&nbsp;&nbsp; or &nbsp;&nbsp;&nbsp; 账号被锁？
              <a href="//home.cathay-ins.com.cn/nologin/password/unlock">
                账号解锁
              </a>
            </div>
          </TabPane>
        </Tabs>
      </Modal>
    );
  }
}
export default LoginModal;
