const path = require('path');
const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const OpenBrowserPlugin = require('open-browser-webpack-plugin');

const dirname = path.resolve(__dirname);
const mockPort = require('./mock/config.js').port;
const devPort = 3011;

module.exports = {
  mode: 'development',
  entry: ['babel-polyfill', './examples/index.js'],
  output: {
    filename: 'bundle.[hash].js',
    path: dirname + '/lib',
    publicPath: '/'
  },
  module: {
    rules: [
      {
        test: /\.css$/,
        use: ['style-loader', 'css-loader']
      },
      {
        test: /\.jsx?$/,
        exclude: /node_modules/,
        use: ['babel-loader']
      },
      {
        test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
        loader: 'url-loader',
        query: {
          limit: 10000,
          name: '[name].[hash:7].[ext]'
        }
      },
      {
        test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
        loader: 'url-loader',
        query: {
          limit: 10000,
          name: '[name].[hash:7].[ext]'
        }
      }
    ]
  },
  resolve: {
    extensions: ['.js', '.jsx', '.json']
  },
  plugins: [
    new webpack.HotModuleReplacementPlugin(),
    new HtmlWebpackPlugin({
      template: dirname + '/examples/index.html',
      hash: false,
      filename: 'index.html',
      inject: 'body',
      minify: {
        collapseWhitespace: true
      }
    }),
    new OpenBrowserPlugin({
      url: 'http://localhost:' + devPort
    })
  ],
  devServer: {
    hot: true,
    host: 'localhost',
    port: devPort,
    disableHostCheck: true,
    historyApiFallback: true,
    proxy: {
      '/**': {
        target: 'http://localhost:' + mockPort
      }
    }
  }
};
